/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    OpenGL/cairo anti-aliasing trick
**
** file:
**    main.c
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, December 2006, placed under the terms of the GPL
**
*******************************************************************************/

#include <gtk/gtk.h>
#include <gtk/gtkgl.h>

#include "events.h"

int
main (int argc,
      char** argv)
{
	GtkWidget* pWindow = NULL;
	GdkGLConfig* pGlConfig = NULL;
	GtkWidget* pDrawingArea = NULL;
	guint uiDrawHandlerId = 0;
	gint iWidth = 0;
	gint iHeight = 0;
 	GError* pError = NULL;

	/* init gtk+, GL and glut */
	gtk_init (&argc, &argv);
	gtk_gl_init (&argc, &argv);

	/* setup GL-context */
	pGlConfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
					       GDK_GL_MODE_ALPHA |
					       GDK_GL_MODE_DEPTH |
					       GDK_GL_MODE_DOUBLE);

	/* check for errors */
	if (!pGlConfig)
	{
		g_print ("Could not setup GL-context!\n");
		return 1;
	}

	/* create window and hook up event-handlers */
	pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	if (!pWindow)
	{
		g_print ("Could not create gtk+-window!\n");
		return 2;
	}

	/* the window only needs handlers for keyboard and close-button */
	g_signal_connect (G_OBJECT (pWindow),
			  "delete-event",
			  G_CALLBACK (delete_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "key-press-event",
			  G_CALLBACK (key_press_handler),
			  NULL);

	/* create drawing-area to be used as OpenGL-context */
	pDrawingArea = gtk_drawing_area_new ();
	if (!pDrawingArea)
	{
		g_print ("Could not create drawing-area!\n");
		return 3;
	}

	/* restrict the initial window-size and connect the GL-context */
	iWidth = 900;
	iHeight = 400;
	gtk_widget_set_size_request (pDrawingArea, iWidth, iHeight);
	gtk_widget_set_gl_capability (pDrawingArea,
				      pGlConfig,
				      NULL,
				      TRUE,
				      GDK_GL_RGBA_TYPE);

	/* drawing-area only needs the three typical handlers for normal GL*/
	g_signal_connect_after (G_OBJECT (pDrawingArea),
				"realize",
				G_CALLBACK (realize_handler),
				NULL);
	g_signal_connect (G_OBJECT (pDrawingArea),
			  "configure-event",
			  G_CALLBACK (configure_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pDrawingArea),
			  "expose-event",
			  G_CALLBACK (expose_handler),
			  NULL);

	/* add the drawing-area and set a few window-properties */
	gtk_container_add (GTK_CONTAINER (pWindow), pDrawingArea);
	gtk_widget_show (pDrawingArea);
	gtk_window_set_icon_from_file (GTK_WINDOW (pWindow),
				       "cairo-scarab.png",
				       &pError);
	gtk_window_set_resizable (GTK_WINDOW (pWindow), TRUE);
	gtk_widget_set_app_paintable (pWindow, FALSE);
	gtk_window_set_decorated (GTK_WINDOW (pWindow), TRUE);
	gtk_widget_show (pWindow);

	/* force a refresh-rate of ~60 Hz */
	uiDrawHandlerId = g_timeout_add (16,
					 (GSourceFunc) draw_handler, 
					 pDrawingArea);

	/* enter event-loop */
	gtk_main ();

	return 0;
}
