/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    OpenGL/cairo anti-aliasing trick
**
** file:
**    opengl.h
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, December 2006, placed under the terms of the GPL
**
*******************************************************************************/

#ifndef _OPENGL_H
#define _OPENGL_H

#include <gtk/gtk.h>
#include <GL/gl.h>

typedef struct _Texture
{
	GLenum   iTarget;
	GLuint	 uiTextureId;
	GLfloat  fWidth;
	GLfloat  fHeight;
	GLuint   iBufferSize;
	GLubyte* pucBufferData; /* free as soon as no longer needed */
} Texture;

GLenum
opengl_what_texture_target (GLuint iWidth,
			    GLuint iHeight);

void
opengl_print_error (gchar* pcFuncName);

void
opengl_enable_vsync (gint iInterval);

void
opengl_change_projection (gboolean bParallel,/* paral. or persp. projection */
			  GLsizei iWidth,    /* current width of GL-context */
			  GLsizei iHeight    /* current height of GL-context*/);

gboolean
opengl_texture_proxy_check (GLenum iTarget, /* GL texture-target to test  */
			    gint iChannels, /* number of color-components */
			    gint iFormat,   /* GL image-format to test    */
			    gint iWidth,    /* width of texture to test   */
			    gint iHeight    /* height of texture to test  */);

GLuint
opengl_upload_texture (GLenum iTarget,    /* GL texture-target to use    */
		       gchar* pcFilename, /* full path/filename of image */
		       gboolean bTrim     /* trim or don't trim edges    */);

Texture*
opengl_upload_text_texture (gchar* pcText,/* text to render into texture    */
			    GLint iWidth, /* width of win. text appears in  */
			    GLint iHeight /* height of win. text appears in */);

void
opengl_paint_checker (GLfloat fStartX, /* x-coord of lower-left corner */
		      GLfloat fStartY, /* y-coord of lower-left corner */
		      GLfloat fSize,   /* width/height of one tile     */
		      GLint iColumns,  /* number of columns to draw    */
		      GLint iRows      /* number of rows to draw       */);

void
opengl_paint_bg (GLfloat fStartX, /* x-coord of lower-left corner */
		 GLfloat fStartY, /* y-coord of lower-left corner */
		 GLfloat fWidth,  /* width of background          */
		 GLfloat fHeight  /* height of background         */);

void
opengl_init (GLint iWidth, /* width of window  */
	     GLint iHeight /* height of window */);

void
opengl_resize (GLsizei iWidth, /* new width to adjust GL-context to  */
	       GLsizei iHeight /* new height to adjust GL-context to */);

void
opengl_draw (GLsizei iWidth, /* current width of GL-context  */
	     GLsizei iHeight /* current height of GL-context */);

void
opengl_cleanup (void);

#endif /* _OPENGL_H */
