/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    OpenGL/cairo anti-aliasing trick
**
** file:
**    shaders.h
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, December 2006, placed under the terms of the GPL
**
*******************************************************************************/

#ifndef _SHADERS_H
#define _SHADERS_H

#include <GL/gl.h>

gboolean
setup_shader_callbacks (void);

GLuint
upload_program (GLenum iTarget,   /* either vertex- or fragment-target    */
		gchar* pcFilename /* full path/filename to shader-program */);

typedef GLvoid
(*glVertexAttrib1sARBFunc) (GLuint index,
			    GLshort x);

typedef GLvoid
(*glVertexAttrib1fARBFunc) (GLuint index,
			    GLfloat x);

typedef GLvoid
(*glVertexAttrib1dARBFunc) (GLuint index,
			    GLdouble x);

typedef GLvoid
(*glVertexAttrib2sARBFunc) (GLuint index,
			    GLshort x,
			    GLshort y);

typedef GLvoid
(*glVertexAttrib2fARBFunc) (GLuint index,
			    GLfloat x,
			    GLfloat y);

typedef GLvoid
(*glVertexAttrib2dARBFunc) (GLuint index,
			    GLdouble x,
			    GLdouble y);

typedef GLvoid
(*glVertexAttrib3sARBFunc) (GLuint index,
			    GLshort x,
			    GLshort y,
			    GLshort z);

typedef GLvoid
(*glVertexAttrib3fARBFunc) (GLuint index,
			    GLfloat x,
			    GLfloat y,
			    GLfloat z);

typedef GLvoid
(*glVertexAttrib3dARBFunc) (GLuint index,
			    GLdouble x,
			    GLdouble y,
			    GLdouble z);

typedef GLvoid
(*glVertexAttrib4sARBFunc) (GLuint index,
			    GLshort x,
			    GLshort y,
			    GLshort z,
			    GLshort w);

typedef GLvoid
(*glVertexAttrib4fARBFunc) (GLuint index,
			    GLfloat x,
			    GLfloat y,
			    GLfloat z,
			    GLfloat w);

typedef GLvoid
(*glVertexAttrib4dARBFunc) (GLuint index,
			    GLdouble x,
			    GLdouble y,
			    GLdouble z,
			    GLdouble w);

typedef GLvoid
(*glVertexAttrib4NubARBFunc) (GLuint index,
			      GLubyte x,
			      GLubyte y,
			      GLubyte z,
			      GLubyte w);

typedef GLvoid
(*glVertexAttrib1svARBFunc) (GLuint index,
			     const GLshort* v);

typedef GLvoid
(*glVertexAttrib1fvARBFunc) (GLuint index,
			     const GLfloat* v);

typedef GLvoid
(*glVertexAttrib1dvARBFunc) (GLuint index,
			     const GLdouble* v);

typedef GLvoid
(*glVertexAttrib2svARBFunc) (GLuint index,
			     const GLshort* v);

typedef GLvoid
(*glVertexAttrib2fvARBFunc) (GLuint index,
			     const GLfloat* v);

typedef GLvoid
(*glVertexAttrib2dvARBFunc) (GLuint index,
			     const GLdouble* v);

typedef GLvoid
(*glVertexAttrib3svARBFunc) (GLuint index,
			     const GLshort* v);

typedef GLvoid
(*glVertexAttrib3fvARBFunc) (GLuint index,
			     const GLfloat* v);

typedef GLvoid
(*glVertexAttrib3dvARBFunc) (GLuint index,
			     const GLdouble* v);

typedef GLvoid
(*glVertexAttrib4bvARBFunc) (GLuint index,
			     const GLbyte* v);

typedef GLvoid
(*glVertexAttrib4svARBFunc) (GLuint index,
			     const GLshort* v);

typedef GLvoid
(*glVertexAttrib4ivARBFunc) (GLuint index,
			     const GLint* v);

typedef GLvoid
(*glVertexAttrib4ubvARBFunc) (GLuint index,
			      const GLubyte* v);

typedef GLvoid
(*glVertexAttrib4usvARBFunc) (GLuint index,
			      const GLushort* v);

typedef GLvoid
(*glVertexAttrib4uivARBFunc) (GLuint index,
			      const GLuint* v);

typedef GLvoid
(*glVertexAttrib4fvARBFunc) (GLuint index,
			     const GLfloat* v);

typedef GLvoid
(*glVertexAttrib4dvARBFunc) (GLuint index,
			     const GLdouble* v);

typedef GLvoid
(*glVertexAttrib4NbvARBFunc) (GLuint index,
			      const GLbyte* v);

typedef GLvoid
(*glVertexAttrib4NsvARBFunc) (GLuint index,
			      const GLshort* v);

typedef GLvoid
(*glVertexAttrib4NivARBFunc) (GLuint index,
			      const GLint* v);

typedef GLvoid
(*glVertexAttrib4NubvARBFunc) (GLuint index,
			       const GLubyte* v);

typedef GLvoid
(*glVertexAttrib4NusvARBFunc) (GLuint index,
			       const GLushort* v);

typedef GLvoid
(*glVertexAttrib4NuivARBFunc) (GLuint index,
			       const GLuint* v);

typedef GLvoid
(*glVertexAttribPointerARBFunc) (GLuint index,
				 GLint size,
				 GLenum type, 
				 GLboolean normalized,
				 GLsizei stride,
				 const GLvoid* pointer);

typedef GLvoid
(*glEnableVertexAttribArrayARBFunc) (GLuint index);

typedef GLvoid
(*glDisableVertexAttribArrayARBFunc) (GLuint index);

typedef GLvoid
(*glProgramStringARBFunc) (GLenum target,
			   GLenum format,
			   GLsizei len, 
			   const GLvoid* string); 

typedef GLvoid
(*glBindProgramARBFunc) (GLenum target,
			 GLuint program);

typedef GLvoid
(*glDeleteProgramsARBFunc) (GLsizei n,
			    const GLuint* programs);

typedef GLvoid
(*glGenProgramsARBFunc) (GLsizei n,
			 GLuint* programs);

typedef GLvoid
(*glProgramEnvParameter4dARBFunc) (GLenum target,
				   GLuint index,
				   GLdouble x,
				   GLdouble y,
				   GLdouble z,
				   GLdouble w);

typedef GLvoid
(*glProgramEnvParameter4dvARBFunc) (GLenum target,
				    GLuint index,
				    const GLdouble* params);

typedef GLvoid
(*glProgramEnvParameter4fARBFunc) (GLenum target,
				   GLuint index,
				   GLfloat x,
				   GLfloat y,
				   GLfloat z,
				   GLfloat w);

typedef GLvoid
(*glProgramEnvParameter4fvARBFunc) (GLenum target,
				    GLuint index,
				    const GLfloat* params);

typedef GLvoid
(*glProgramLocalParameter4dARBFunc) (GLenum target,
				     GLuint index,
				     GLdouble x,
				     GLdouble y,
				     GLdouble z,
				     GLdouble w);
typedef GLvoid
(*glProgramLocalParameter4dvARBFunc) (GLenum target,
				      GLuint index,
				      const GLdouble* params);
typedef GLvoid
(*glProgramLocalParameter4fARBFunc) (GLenum target,
				     GLuint index,
				     GLfloat x,
				     GLfloat y,
				     GLfloat z,
				     GLfloat w);

typedef GLvoid
(*glProgramLocalParameter4fvARBFunc) (GLenum target,
				      GLuint index,
				      const GLfloat* params);

typedef GLvoid
(*glGetProgramEnvParameterdvARBFunc) (GLenum target,
				      GLuint index,
				      GLdouble* params);

typedef GLvoid
(*glGetProgramEnvParameterfvARBFunc) (GLenum target,
				      GLuint index, 
				      GLfloat* params);

typedef GLvoid
(*glGetProgramLocalParameterdvARBFunc) (GLenum target,
					GLuint index,
					GLdouble* params);

typedef GLvoid
(*glGetProgramLocalParameterfvARBFunc) (GLenum target,
					GLuint index, 
					GLfloat* params);

typedef GLvoid
(*glGetProgramivARBFunc) (GLenum target,
			  GLenum pname,
			  GLint* params);

typedef GLvoid
(*glGetProgramStringARBFunc) (GLenum target,
			      GLenum pname,
			      GLvoid* string);

typedef GLvoid
(*glGetVertexAttribdvARBFunc) (GLuint index,
			       GLenum pname,
			       GLdouble* params);

typedef GLvoid
(*glGetVertexAttribfvARBFunc) (GLuint index,
			       GLenum pname,
			       GLfloat* params);

typedef GLvoid
(*glGetVertexAttribivARBFunc) (GLuint index,
			       GLenum pname,
			       GLint* params);

typedef GLvoid
(*glGetVertexAttribPointervARBFunc) (GLuint index,
				     GLenum pname,
				     GLvoid** pointer);

typedef GLboolean
(*glIsProgramARBFunc) (GLuint program);

extern glVertexAttrib1sARBFunc glVertexAttrib1sARB;
extern glVertexAttrib1fARBFunc glVertexAttrib1fARB;
extern glVertexAttrib1dARBFunc glVertexAttrib1dARB;
extern glVertexAttrib2sARBFunc glVertexAttrib2sARB;
extern glVertexAttrib2fARBFunc glVertexAttrib2fARB;
extern glVertexAttrib2dARBFunc glVertexAttrib2dARB;
extern glVertexAttrib3sARBFunc glVertexAttrib3sARB;
extern glVertexAttrib3fARBFunc glVertexAttrib3fARB;
extern glVertexAttrib3dARBFunc glVertexAttrib3dARB;
extern glVertexAttrib4sARBFunc glVertexAttrib4sARB;
extern glVertexAttrib4fARBFunc glVertexAttrib4fARB;
extern glVertexAttrib4dARBFunc glVertexAttrib4dARB;
extern glVertexAttrib4NubARBFunc glVertexAttrib4NubARB;
extern glVertexAttrib1svARBFunc glVertexAttrib1svARB;
extern glVertexAttrib1fvARBFunc glVertexAttrib1fvARB;
extern glVertexAttrib1dvARBFunc glVertexAttrib1dvARB;
extern glVertexAttrib2svARBFunc glVertexAttrib2svARB;
extern glVertexAttrib2fvARBFunc glVertexAttrib2fvARB;
extern glVertexAttrib2dvARBFunc glVertexAttrib2dvARB;
extern glVertexAttrib3svARBFunc glVertexAttrib3svARB;
extern glVertexAttrib3fvARBFunc glVertexAttrib3fvARB;
extern glVertexAttrib3dvARBFunc glVertexAttrib3dvARB;
extern glVertexAttrib4bvARBFunc glVertexAttrib4bvARB;
extern glVertexAttrib4svARBFunc glVertexAttrib4svARB;
extern glVertexAttrib4ivARBFunc glVertexAttrib4ivARB;
extern glVertexAttrib4ubvARBFunc glVertexAttrib4ubvARB;
extern glVertexAttrib4usvARBFunc glVertexAttrib4usvARB;
extern glVertexAttrib4uivARBFunc glVertexAttrib4uivARB;
extern glVertexAttrib4fvARBFunc glVertexAttrib4fvARB;
extern glVertexAttrib4dvARBFunc glVertexAttrib4dvARB;
extern glVertexAttrib4NbvARBFunc glVertexAttrib4NbvARB;
extern glVertexAttrib4NsvARBFunc glVertexAttrib4NsvARB;
extern glVertexAttrib4NivARBFunc glVertexAttrib4NivARB;
extern glVertexAttrib4NubvARBFunc glVertexAttrib4NubvARB;
extern glVertexAttrib4NusvARBFunc glVertexAttrib4NusvARB;
extern glVertexAttrib4NuivARBFunc glVertexAttrib4NuivARB;
extern glVertexAttribPointerARBFunc glVertexAttribPointerARB;
extern glEnableVertexAttribArrayARBFunc glEnableVertexAttribArrayARB;
extern glDisableVertexAttribArrayARBFunc glDisableVertexAttribArrayARB;
extern glProgramStringARBFunc glProgramStringARB;
extern glBindProgramARBFunc glBindProgramARB;
extern glDeleteProgramsARBFunc glDeleteProgramsARB;
extern glGenProgramsARBFunc glGenProgramsARB;
extern glProgramEnvParameter4dARBFunc glProgramEnvParameter4dARB;
extern glProgramEnvParameter4dvARBFunc glProgramEnvParameter4dvARB;
extern glProgramEnvParameter4fARBFunc glProgramEnvParameter4fARB;
extern glProgramEnvParameter4fvARBFunc glProgramEnvParameter4fvARB;
extern glProgramLocalParameter4dARBFunc glProgramLocalParameter4dARB;
extern glProgramLocalParameter4dvARBFunc glProgramLocalParameter4dvARB;
extern glProgramLocalParameter4fARBFunc glProgramLocalParameter4fARB;
extern glProgramLocalParameter4fvARBFunc glProgramLocalParameter4fvARB;
extern glGetProgramEnvParameterdvARBFunc glGetProgramEnvParameterdvARB;
extern glGetProgramEnvParameterfvARBFunc glGetProgramEnvParameterfvARB;
extern glGetProgramLocalParameterdvARBFunc glGetProgramLocalParameterdvARB;
extern glGetProgramLocalParameterfvARBFunc glGetProgramLocalParameterfvARB;
extern glGetProgramivARBFunc glGetProgramivARB;
extern glGetProgramStringARBFunc glGetProgramStringARB;
extern glGetVertexAttribdvARBFunc glGetVertexAttribdvARB;
extern glGetVertexAttribfvARBFunc glGetVertexAttribfvARB;
extern glGetVertexAttribivARBFunc glGetVertexAttribivARB;
extern glGetVertexAttribPointervARBFunc glGetVertexAttribPointervARB;
extern glIsProgramARBFunc glIsProgramARB;

#endif /* _SHADERS_H */
